# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/weblogic/provider'

Puppet::Type.type(:wls_opatch).provide(:prefetching) do
  include EasyType::Provider

  desc 'Apply Oracle OPatches'

  mk_resource_methods

  def self.prefetch(resources)
    os_user = check_single_os_user(resources)
    orainst_dir = check_single_orainst_dir(resources)

    all_patches = oracle_homes(resources).collect { |h| patches_in_home(h, os_user, orainst_dir) }.flatten
    resources.each_key do |patch_name|
      resources[patch_name].provider = new(:name => name, :ensure => :present) if all_patches.include?(patch_name)
    end
  end

  # Emits a log message at the log level specified for the associated resource.
  # The log entry is associated with this property.
  # @param msg [String] the message to log
  # @return [void]
  #
  def log(msg)
    Puppet::Util::Log.create(
      :level => resource[:loglevel],
      :message => msg,
      :source => self
    )
  end

  def opatch(command, _options = {})
    oracle_product_home_dir = resource[:oracle_product_home_dir]
    jre_specified = resource[:jdk_home_dir] ? " -jre #{resource[:jdk_home_dir]} " : ''
    orainst                 = "-invPtrLoc #{resource[:orainst_dir]}/oraInst.loc "
    os_user                 = resource[:os_user]
    full_command            = "#{oracle_product_home_dir}/OPatch/opatch #{command} -oh #{oracle_product_home_dir} #{jre_specified} #{orainst}"
    output = Puppet::Util::Execution.execute(full_command, :failonfail => true, :uid => os_user)
    if resource.logoutput.to_s == 'true'
      output.split("\n").each { |line| log(line) }
    else
      Puppet.debug output
    end
    output
  end

  def remove_incompatible_patches
    oracle_product_home_dir = resource[:oracle_product_home_dir]
    fact = Facter.value(:wls_install_homes) || {}
    @installed_patches ||= fact.dig(oracle_product_home_dir, 'installed_patches') || []
    resource[:remove_patches_before].each do |remove_patch_id|
      if @installed_patches.include?(remove_patch_id) || force_remove
        Puppet.debug "Incompatible patch #{remove_patch_id} will be removed from #{oracle_product_home_dir}."
        begin
          opatch("rollback -id #{remove_patch_id} -silent ")
        rescue StandardError
          raise unless force_remove

          Puppet.debug "uninstall of patch #{remove_patch_id} failed, but we are continuing..."
        end
        #
        # because multiple patches can have the same incompatible patch, make sure
        # it is only removed once.
        #
        @installed_patches.delete(remove_patch_id)
        #
        # Now see if any of the incompatible patches named in remove_patches_before
        # is already available. If so, we make sure we change the provide so it
        # is no longer installed. The removal of incompatible patches doesn't need
        # the provider. But if you want to re-apply the patch after the removal,
        # the provider must report it absent
        #
        title_name = "#{oracle_product_home_dir}:#{remove_patch_id}"
        catalog_resource = resource.catalog.resource(:wls_opatch, title_name)
        catalog_resource.provider = self.class.new(:name => title_name, :ensure => :absent) if catalog_resource
      else
        Puppet.debug "Incompatible patch #{remove_patch_id} not found in #{oracle_product_home_dir}."
      end
    end
  end

  private

  def installed_patches
    opatch('lsinventory').scan(/Patch\s.(\d+)\s.*:\sapplied on/).flatten
  end

  def self.patches_in_home(oracle_product_home_dir, os_user, orainst_dir)
    full_command = "#{oracle_product_home_dir}/OPatch/opatch lsinventory -oh #{oracle_product_home_dir} -invPtrLoc #{orainst_dir}/oraInst.loc"
    raw_list = Puppet::Util::Execution.execute(full_command, :failonfail => true, :uid => os_user)
    Puppet.debug raw_list
    patch_ids = raw_list.scan(/Patch\s.(\d+)\s.*:\sapplied on/).flatten
    patch_ids.collect { |p| "#{oracle_product_home_dir}:#{p}" }
  end

  def self.check_single_os_user(resources)
    os_users = resources.map { |_k, v| v.os_user }.uniq
    fail "wls_opatch doesn't support multiple os_users" if os_users.size > 1

    os_users.first
  end

  def self.check_single_orainst_dir(resources)
    orainst_dir = resources.map { |_k, v| v.orainst_dir }.uniq
    fail "wls_opatch doesn't support multiple orainst_dir" if orainst_dir.size > 1

    orainst_dir.first
  end

  def self.oracle_homes(resources)
    resources.map { |_k, v| v.oracle_product_home_dir }.uniq
  end
end
