# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:security_remote_anonymous_rmi_t3_enabled, :parent => Puppet::Parameter::WlsProperty) do
  extend  Puppet_X::EnterpriseModules::WebLogic::DomainPaths # Changes the actual path for domain resource
  include ::Puppet_X::EnterpriseModules::WebLogic::Types::NumberedBoolean
  include ::Puppet_X::EnterpriseModules::WebLogic::Access

  desc <<-DESC
  Returns true if remote anonymous RMI access via T3 is permitted. If remote anonymous RMI access is not allowed, then client requests that do not specify a username / password may fail.
  An example on how to use this:

      wls_domain {a_wls_domain :
         ...
         security_remote_anonymous_rmi_t3_enabled => '1'
         ...
      }

  This is an extended property. Before you can use it add it to the `wls_settings` property `extra_properties`.

      wls_setting{'domain':
         ...
        extra_properties => ['wls_domain:security_remote_anonymous_rmi_t3_enabled']
         ...
      }

  This help text generated from MBean text of the WebLogic server.

  DESC

  #
  # Setting this value sometimes reults in a disconnect, leaving The daemon clueless. There fore
  # This special code. When a disconnect occors, we reconnect and put the daemon back in the edit mode
  # so the other code will see the state they expect
  #
  on_apply do
    domain_values = configuration[resource.domain]
    weblogic_user        = domain_values['weblogic_user'] || 'weblogic'
    weblogic_connect_url = domain_values['connect_url'] || 't3://localhost:7001'
    weblogic_password    = domain_values['weblogic_password'].nil? ? '' : decrypted_value(domain_values['weblogic_password'])
    actual_paths(resource.to_hash).each do |path|
      Puppet.debug wlst template('wls_config/wls_domain/set_remote_anonymous_rmi_t3_enabled.erb', binding)
    end
  end

  data_type("Optional[Variant[Enum['absent'],Integer[0,1]]]")

  maps_to('SecurityConfiguration/%<domain_name>s/RemoteAnonymousRMIT3Enabled') {}
end
