# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

module Puppet
  Type.newtype(:wls_saf_imported_destination) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-DESC
    This resource allows you to manage a SAF imported destinations in a JMS Module of a WebLogic domain.

    Here is an example on how you should use this:

        wls_saf_imported_destination { 'jmsClusterModule:SAFImportedDestinations-0':
          ensure               => 'present',
          defaulttargeting     => 1,
          errorhandling        => 'ErrorHandling-0',
          jndiprefix           => 'saf_',
          remotecontext        => 'RemoteSAFContext-0',
          timetolivedefault    => '1000000000',
          usetimetolivedefault => 1,,
        }

    In this example you are managing a SAF imported destination in the default domain. When you want to
    manage a SAF imported destination in a specific domain, you can use:

        wls_saf_imported_destination { 'my_domain/jmsClusterModule:SAFImportedDestinations-1':
          ensure               => 'present',
          defaulttargeting     => '0',
          jndiprefix           => 'saf2_',
          remotecontext        => 'RemoteSAFContext-1',
          subdeployment        => 'safServers',
          usetimetolivedefault => '0',
        }

    DESC

    bean_path('/JMSSystemResources/%<jmsmodule>s/JMSResource/%<jmsmodule>s/SAFImportedDestinations')
    bean_type('SAFImportedDestinations')

    ensurable
    parameter :imported_destination_name
    parameter :domain
    parameter :name
    parameter :jmsmodule
    parameter :timeout

    property :errorhandling
    property :remotecontext
    property :jndiprefix
    property :timetolivedefault
    property :usetimetolivedefault
    property :defaulttargeting
    property :subdeployment
    extraproperties do
      property :default_targeting_enabled
      property :jndi_prefix
      property :notes
      property :saf_error_handling
      property :saf_remote_context
      property :sub_deployment_name
      property :time_to_live_default
      property :unit_of_order_routing
      property :use_saf_time_to_live_default
      #
      # Weblogic 12.2.1 new properties
      #
    end

    add_title_attributes(:jmsmodule, :imported_destination_name) do
      /(.+):(.+)/
    end
  end
end
